@@ -5,9 +5,9 @@
 #' and optionally delete GitHub's default labels (if `delete_default = TRUE`).
 #' It will never delete labels that have associated issues.
 #'
-#' `use_tidy_labels()` calls `use_github_labels()` with tidyverse conventions
-#' powered by `tidy_labels()`, `tidy_labels_rename()`, `tidy_label_colours()`
-#' and `tidy_label_descriptions()`.
+#' `use_tidy_github_labels()` calls `use_github_labels()` with tidyverse
+#' conventions powered by `tidy_labels()`, `tidy_labels_rename()`,
+#' `tidy_label_colours()` and `tidy_label_descriptions()`.
 #'
 #' @section Label usage:
 #' Labels are used as part of the issue-triage process, designed to minimise the
@@ -22,10 +22,10 @@
 #' * `good first issue` indicates a good issue for first-time contributors.
 #' * `help wanted` indicates that a maintainer wants help on an issue.
 #'
-#' @param repo_spec,host,auth_token \lifecycle{defunct}: These arguments are
-#'   now deprecated and will be removed in the future. Any input provided via
-#'   these arguments is not used. The target repo, host, and auth token are all
-#'   now determined from the current project's Git remotes.
+#' @param repo_spec,host,auth_token `r lifecycle::badge("deprecated")`: These
+#'   arguments are now deprecated and will be removed in the future. Any input
+#'   provided via these arguments is not used. The target repo, host, and auth
+#'   token are all now determined from the current project's Git remotes.
 #' @param labels A character vector giving labels to add.
 #' @param rename A named vector with names giving old names and values giving
 #'   new names.
@@ -221,19 +221,7 @@ use_github_labels <- function(repo_spec = deprecated(),
 
 #' @export
 #' @rdname use_github_labels
-use_tidy_labels <- function(repo_spec = deprecated(),
-                            host = deprecated(),
-                            auth_token = deprecated()) {
-  if (lifecycle::is_present(repo_spec)) {
-    deprecate_warn_repo_spec("use_tidy_labels")
-  }
-  if (lifecycle::is_present(host)) {
-    deprecate_warn_host("use_tidy_labels")
-  }
-  if (lifecycle::is_present(auth_token)) {
-    deprecate_warn_auth_token("use_tidy_labels")
-  }
-
+use_tidy_github_labels <- function() {
   use_github_labels(
     labels = tidy_labels(),
     rename = tidy_labels_rename(),
@@ -272,6 +260,7 @@ tidy_label_colours <- function() {
     "bug" = "E0B3A2",
     "documentation" = "CBBAB8",
     "feature" = "B4C3AE",
+    "upkeep" = "C2ACC0",
     "good first issue :heart:" = "CBBAB8",
     "help wanted :heart:" = "C5C295",
     "reprex" = "C5C295",
@@ -285,6 +274,7 @@ tidy_label_descriptions <- function() {
   c(
     "bug" = "an unexpected problem or unintended behavior",
     "feature" = "a feature request or enhancement",
+    "upkeep" = "maintenance, infrastructure, and similar",
     "reprex" = "needs a minimal reproducible example",
     "wip" = "work in progress",
     "documentation" = "",
