@@ -2,7 +2,8 @@
 #'
 #' @description `use_version()` increments the "Version" field in `DESCRIPTION`,
 #'   adds a new heading to `NEWS.md` (if it exists), and commits those changes
-#'   (if package uses Git).
+#'   (if package uses Git). It makes the same update to a line like `PKG_version
+#'   = "x.y.z";` in `src/version.c` (if it exists).
 #'
 #' @description `use_dev_version()` increments to a development version, e.g.
 #'   from 1.0.0 to 1.0.0.9000. If the existing version is already a development
@@ -32,14 +33,16 @@ NULL
 #' @rdname use_version
 #' @export
 use_version <- function(which = NULL) {
-  if (is.null(which) && !interactive()) {
+  if (is.null(which) && !is_interactive()) {
     return(invisible(FALSE))
   }
 
   check_is_package("use_version()")
-  check_uncommitted_changes()
+  challenge_uncommitted_changes(
+    msg = "There are uncommitted changes and you're about to bump version"
+  )
 
-  new_ver <- choose_version(which)
+  new_ver <- choose_version("What should the new version be?", which)
   if (is.null(new_ver)) {
     return(invisible(FALSE))
   }
@@ -51,7 +54,13 @@ use_version <- function(which = NULL) {
     use_news_heading(new_ver)
   }
 
-  git_ask_commit("Increment version number")
+  use_c_version(new_ver)
+
+  git_ask_commit(
+    glue("Increment version number to {new_ver}"),
+    untracked = TRUE,
+    paths = c("DESCRIPTION", "NEWS.md", path("src", "version.c"))
+  )
   invisible(TRUE)
 }
 
@@ -66,7 +75,7 @@ use_dev_version <- function() {
   use_version(which = "dev")
 }
 
-choose_version <- function(which = NULL) {
+choose_version <- function(message, which = NULL) {
   ver <- desc::desc_get_version(proj_get())
   versions <- bump_version(ver)
 
@@ -76,7 +85,8 @@ choose_version <- function(which = NULL) {
         "{format(names(versions), justify = 'right')} --> {versions}"
       ),
       title = glue(
-        "Current version is {ver}.\n", "Which part to increment? (0 to exit)"
+        "Current version is {ver}.\n",
+        "{message} (0 to exit)"
       )
     )
     if (choice == 0) {
@@ -99,3 +109,22 @@ bump_ <- function(x, ver) {
   d <- desc::desc(text = paste0("Version: ", ver))
   suppressMessages(d$bump_version(x)$get("Version")[[1]])
 }
+
+use_c_version <- function(ver) {
+  version_path <- proj_path("src", "version.c")
+
+  if (!file_exists(version_path)) {
+    return()
+  }
+
+  hint <- glue("{project_name()}_version")
+  ui_done("
+    Setting {ui_field(hint)} to {ui_value(ver)} in {ui_path(version_path)}")
+
+  lines <- read_utf8(version_path)
+
+  re <- glue("(^.*{project_name()}_version = \")([0-9.]+)(\";$)")
+  lines <- gsub(re, glue("\\1{ver}\\3"), lines)
+
+  write_utf8(version_path, lines)
+}
